==> fferrorhandler-1.0.30.tgz: package/index.js <==
'use strict';

const logger = require('@firstfleet/fflogger');
const { PostToSlack } = require('@firstfleet/ffslack');
/**
 * An object that holds error levels.
 * @type {{COMMON: string, FATAL: string, CRITICAL: string}}
 */
const errorLevels = {
    COMMON: 'common',
    CRITICAL: 'critical',
    FATAL: 'fatal'
};

/**
 * Creates a new AppError
 * @class
 */
class AppError {
    constructor(sender, method, message, severity, isOperational, channelId, sendToSlack) {
        Error.call(this);
        Error.captureStackTrace(this);
        this.sender = sender;
        this.method = method;
        this.message = message;
        this.severity = severity;
        this.isOperational = isOperational;
        this.channelId = channelId;
        this.sendToSlack = sendToSlack;
    }
}

/**
 * Attach error prototypical inheritance
 */
AppError.prototype.__proto__ = Error.prototype;

const logError = (error) => {
    return new Promise((resolve, reject) => {
        try {
            logger.error(error.sender, error.method, 'Error: ', error);
            if (error.severity === errorLevels.CRITICAL || error.sendToSlack) {
                PostToSlack(error.sender, error.message, error.channelId, error.stack);
            }
            resolve('Error Logged')
        } catch(err) {
            reject(err)
        }
    });
};

const logUncaughtError = (error) => {
    return new Promise((resolve, reject) => {
        try {
            logger.error('Uncaught Error', 'Global Error Catch', 'Error: ', error);
            resolve(error)
        } catch(err) {
            reject(error)
        }
    });
};

/**
 * Handles application errors and calls the ffLogger (sends log to console if not
 * in production and also to papertrail). Collects params and stack trace.
 * @param sender - Machine sending the error
 * @param method - method sending the error
 * @param message - error description
 * @param severity - which errorLevel (defined in error handler)
 * @param isOperational - is it an operational error or programmer error
 * @param channelId - slack channel to post to if error is critical
 * @param sendToSlack - should the message be sent to slack, even if not critical
 * @returns {Promise<any>}
 */
const handleError = (sender, method, message, severity, isOperational, channelId = 'alerts_critical', sendToSlack = false) => {
    return new Promise((resolve, reject) => {
        const error = new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);
        return logError(error)
            .then(error => resolve(error))
            .catch(caughtError => {
                console.error(caughtError);
                reject(caughtError)
            })
    });
};

/**
 * Logs error message to papertrail and optionally posts to Slack
 * @param {string} appName - Name of the application (ex: NodeJobsConsumer)
 * @param {string} methodName - Name of method (ex: AuditCard)
 * @param {string} message - Error message (just the error, no distinct details)
 * @param {string} messageDetail - Details or stack trace of the error (ex: DriverID - BEAB)
 * @param {string} [slackChannel] - If Slack post desired, this channel will be used
 */
const logAndNotify = (appName, methodName, message, messageDetail, slackChannel) => {
    try {
        logger.error(appName, methodName, `Error: ${message}\n${messageDetail}`, '');
        if (slackChannel){
            PostToSlack(appName, `${methodName}-${message}`, slackChannel, messageDetail);
        }
    } catch (error) {
        console.error(error.message);
    }
};

const createError = (sender = 'errorhandler', method = 'default method', message = 'default error message', severity = errorLevels.COMMON, isOperational = false, channelId = 'alerts_critical', sendToSlack = 'false') => {
    return new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);
};

const handleErrorList = (errorList) => {
    let finalErrorList = []; //new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);

    //Loop over list of  error candidates
    errorList.map(error => {
        let errorExists = false;

        //Loop over existing errors to see if we've seen this one yet
        finalErrorList.forEach(finalError => {
            if (finalError.messsage === error.message){  //If found, increment the counter on that one
                errorExists = true;
                finalError.errorCount = finalError.errorCount + 1;
                return error
            }
        });

        //If it's a new one, add it to the final distinct array.
        if (!errorExists){
            finalErrorList.push(Object.assign(error, {'errorCount': 0}))
        }
    });

    //Send distinct error list
    finalErrorList.forEach(error => {
       logError(error);
    })
};

/**
 * Takes in a custom error object and return a boolean. If the error
 * is an operation error it returns true, and returns false if it is a
 * Programmer error.
 * @param error
 * @returns {*}
 */
const isTrustedError = (error) => {
    return error.isOperational;
};

/**
 * Global handler for all uncaught exceptions
 * If an uncaughtException is not an Operational error,
 * gracefully restart the application.
 */
process.on('uncaughtException', (error) => {
    logUncaughtError(error)
        .then(_error => {
            if (!isTrustedError(_error)) {
                process.exit(1)
            }
        })
        .catch(caughtError => {
            console.error(error);
            process.exit(1)
        })

});

/**
 * Global handler for all uncaught promise rejections.
 * If an uncaught promise rejection is thrown this will handle the error
 * gracefully.
 */
process.on('unhandledRejection', error => {
    logUncaughtError(error);
});

module.exports = {
    AppError,
    handleError,
    isTrustedError,
    errorLevels,
    PostToSlack,
    handleErrorList,
    createError,
    logAndNotify
};

==> fferrorhandler-1.0.31.tgz: package/index.js <==
'use strict';

const logger = require('@firstfleet/fflogger');
const { PostToSlack } = require('@firstfleet/ffslack');
/**
 * An object that holds error levels.
 * @type {{COMMON: string, FATAL: string, CRITICAL: string}}
 */
const errorLevels = {
    COMMON: 'common',
    CRITICAL: 'critical',
    FATAL: 'fatal'
};

/**
 * Creates a new AppError
 * @class
 */
class AppError {
    constructor(sender, method, message, severity, isOperational, channelId, sendToSlack) {
        Error.call(this);
        Error.captureStackTrace(this);
        this.sender = sender;
        this.method = method;
        this.message = message;
        this.severity = severity;
        this.isOperational = isOperational;
        this.channelId = channelId;
        this.sendToSlack = sendToSlack;
    }
}

/**
 * Attach error prototypical inheritance
 */
AppError.prototype.__proto__ = Error.prototype;

const logError = (error) => {
    return new Promise((resolve, reject) => {
        try {
            logger.error(error.sender, error.method, 'Error: ', error);
            if (error.severity === errorLevels.CRITICAL || error.sendToSlack) {
                PostToSlack(error.sender, error.message, error.channelId, error.stack);
            }
            resolve('Error Logged')
        } catch(err) {
            reject(err)
        }
    });
};

const logUncaughtError = (error) => {
    return new Promise((resolve, reject) => {
        try {
            logger.error('Uncaught Error', 'Global Error Catch', `Error: ${error.message}`);
            try {
                PostToSlack(process.env.PAPERTRAIL_PROGRAM || 'Unknown App', `Global error catch - ${error.message}`, 'alerts_warning', error.stack);
            } catch (error) {
                console.log(error.message);
            }
            resolve(error)
        } catch(err) {
            reject(error)
        }
    });
};

/**
 * Handles application errors and calls the ffLogger (sends log to console if not
 * in production and also to papertrail). Collects params and stack trace.
 * @param sender - Machine sending the error
 * @param method - method sending the error
 * @param message - error description
 * @param severity - which errorLevel (defined in error handler)
 * @param isOperational - is it an operational error or programmer error
 * @param channelId - slack channel to post to if error is critical
 * @param sendToSlack - should the message be sent to slack, even if not critical
 * @returns {Promise<any>}
 */
const handleError = (sender, method, message, severity, isOperational, channelId = 'alerts_critical', sendToSlack = false) => {
    return new Promise((resolve, reject) => {
        const error = new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);
        return logError(error)
            .then(error => resolve(error))
            .catch(caughtError => {
                console.error(caughtError);
                reject(caughtError)
            })
    });
};

/**
 * Logs error message to papertrail and optionally posts to Slack
 * @param {string} appName - Name of the application (ex: NodeJobsConsumer)
 * @param {string} methodName - Name of method (ex: AuditCard)
 * @param {string} message - Error message (just the error, no distinct details)
 * @param {string} messageDetail - Details or stack trace of the error (ex: DriverID - BEAB)
 * @param {string} [slackChannel] - If Slack post desired, this channel will be used
 */
const logAndNotify = (appName, methodName, message, messageDetail, slackChannel) => {
    try {
        logger.error(appName, methodName, `Error: ${message}\n${messageDetail}`, '');
        if (slackChannel){
            PostToSlack(appName, `${methodName}-${message}`, slackChannel, messageDetail);
        }
    } catch (error) {
        console.error(error.message);
    }
};

const createError = (sender = 'errorhandler', method = 'default method', message = 'default error message', severity = errorLevels.COMMON, isOperational = false, channelId = 'alerts_critical', sendToSlack = 'false') => {
    return new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);
};

const handleErrorList = (errorList) => {
    let finalErrorList = []; //new AppError(sender, method, message, severity, isOperational, channelId, sendToSlack);

    //Loop over list of  error candidates
    errorList.map(error => {
        let errorExists = false;

        //Loop over existing errors to see if we've seen this one yet
        finalErrorList.forEach(finalError => {
            if (finalError.messsage === error.message){  //If found, increment the counter on that one
                errorExists = true;
                finalError.errorCount = finalError.errorCount + 1;
                return error
            }
        });

        //If it's a new one, add it to the final distinct array.
        if (!errorExists){
            finalErrorList.push(Object.assign(error, {'errorCount': 0}))
        }
    });

    //Send distinct error list
    finalErrorList.forEach(error => {
       logError(error);
    })
};

/**
 * Takes in a custom error object and return a boolean. If the error
 * is an operation error it returns true, and returns false if it is a
 * Programmer error.
 * @param error
 * @returns {*}
 */
const isTrustedError = (error) => {
    return error.isOperational;
};

/**
 * Global handler for all uncaught exceptions
 * If an uncaughtException is not an Operational error,
 * gracefully restart the application.
 */
process.on('uncaughtException', (error) => {
    logUncaughtError(error)
        .then(_error => {
            if (!isTrustedError(_error)) {
                process.exit(1)
            }
        })
        .catch(caughtError => {
            console.error(error);
            process.exit(1)
        })

});

/**
 * Global handler for all uncaught promise rejections.
 * If an uncaught promise rejection is thrown this will handle the error
 * gracefully.
 */
process.on('unhandledRejection', error => {
    logUncaughtError(error);
});

module.exports = {
    AppError,
    handleError,
    isTrustedError,
    errorLevels,
    PostToSlack,
    handleErrorList,
    createError,
    logAndNotify
};
